/***********************************************************

             PNM image input/output functions

 ***********************************************************/
  #include <stdio.h>
  #include <stdlib.h>
  #include <math.h>
  #include <string.h>

/*=======================================================
  -------------------------------------------------------
  Structure for an image size
  =======================================================*/
typedef struct _point {
  int	x, y;
} Point;

/*=======================================================
  -------------------------------------------------------
  Structure for an image store memory
  =======================================================*/
typedef struct _image {
  int		type;	    /* Type of iamge  1: gray, 3: color */
  Point		size;	    /* Size of iamge */
  unsigned char *data ;	/* Image data */
} Image;

/*=======================================================
  -------------------------------------------------------
  Generate an image store memory
  -------------------------------------------------------
  Argument
      int x: x size (width of image)
      int y: y size (height of image)
	  int m: channel number
  -------------------------------------------------------
  Return Value
      Image *: pointer to image memory structure
  =======================================================*/
Image *new_image( int x, int y, int m )
{
  Image	*a;

  a = (Image *)calloc( 1, sizeof( Image ) );
  if( a == NULL ) exit( -1 );
  a->type = m;
  a->size.x = x;
  a->size.y = y;
  a->data = (unsigned char *)calloc(m * x * y, sizeof(unsigned char)) ;
  if( a->data == NULL ) exit( -1 );

  return( a );
}

/*=======================================================
  -------------------------------------------------------
  Release an image store memory
  -------------------------------------------------------
  Argument
      Image *a: pointer to image memory structure
  -------------------------------------------------------
  Return Value
      No
  =======================================================*/
void free_image( Image *a )
{
  if( a != NULL ){
    if( a->data != NULL ) free( a->data );
    free( a );
  }
}
  

/*=======================================================
  -------------------------------------------------------
  Return the type of PNM image
  -------------------------------------------------------
  Argument
      FILE *fp: pointer to FILE structure
  -------------------------------------------------------
  Return Value
      int : type of image ( 5 or 6 )
  =======================================================*/
int image_type( FILE *fp )
{
    int         type;
    char        s[80];

    if( ungetc(getc(fp), fp) == EOF )           exit( -1 );

    do{
	if( fgets( s, sizeof(s), fp ) == NULL ) exit( -1 );
    }while( s[0]=='#' );

    if( s[0]!='P' ) exit( -1 );

    type = atoi(&s[1]) ;
    return( type );
}


/*=======================================================
  -------------------------------------------------------
  Return the size of PNM image
  -------------------------------------------------------
  Argument
      FILE *fp: pointer to FILE structure
      int x: x size (width of image)
      int y: y size (height of image)
  -------------------------------------------------------
  Return Value
      int 1: success  -1: failure
  =======================================================*/
int read_size( FILE *fp, int *x, int *y ) 
{
    char	s[255];
    int		rx, ry;

    do{
	if( fgets( s, sizeof(s), fp ) == NULL )	exit( -1 );
    }while( s[0]=='#' );

    rx = atoi( s );		if( x != NULL ) *x = rx;
    ry = atoi(strchr(s,' '));	if( y != NULL ) *y = ry;

    if( rx <= 0 || ry <= 0 ) exit( -1 );
    return( 1 );
}

/*=======================================================
  -------------------------------------------------------
	Return the maximum luminance of PNM image
  -------------------------------------------------------
  Argument
      FILE *fp: pointer to FILE structure
  -------------------------------------------------------
  Return Value
      int : maximum luminance
=======================================================*/
int read_maxi( FILE *fp ) 
{
    char	s[255];
    int		l;
    
    do{
	if( fgets( s, sizeof(s), fp ) == NULL ) exit( -1 );
    }while( s[0]=='#' );

    l = atoi( s );
    return( l );
}
    
/*=======================================================
  -------------------------------------------------------
  PNM image input (double)
  -------------------------------------------------------
  Argument
      char *file: input file name
  -------------------------------------------------------
  Return Value
      sImage *: Pointer to image memory structure
  =======================================================*/
Image *pnmread( char *file )
{
    unsigned char	*buff,*b1;
	unsigned char	*b2;
    FILE     		*fp;
    int      		i, j, x, y, m, type;
    Image		*img;

    if ((fp=fopen(file,"rb")) == NULL) exit( -1 );
    type = image_type( fp );
    read_size( fp, &x, &y );
    if ( read_maxi( fp ) != 255 )	   exit( -1 );

    if ( type == 5 ){
      m = 1;
      fprintf(stderr, "Reading PGM format, %d x %d image : %s\n", x, y, file);
    } else if( type == 6 ){
      m = 3;
      fprintf(stderr, "Reading PPM format, %d x %d image : %s\n", x, y, file);
    } else exit( -1 );

    img = new_image( x, y, m );

    buff = (unsigned char *)calloc( m * x * y, sizeof( char ) );
    if( buff == NULL ) exit( -1 );

    fread(buff, 1, m * x * y, fp);
    b2 = img->data;
    b1 = buff;
    for ( i=0; i<y; i++ ){
      for ( j=0; j<m*x ;j++ ){
		  *b2++ = *b1++;
      }
    }
		
    fclose(fp);
    free(buff);

    return( img );
}

/*=======================================================
  -------------------------------------------------------
	PNM image output
  -------------------------------------------------------
  Argument
      Image *img: Pointer to image memory structure for PPM
  -------------------------------------------------------
  Return Value
      char *file: input file name
  =======================================================*/
void pnmwrite( Image *img, char *file )
{
    unsigned char 	*buff, *b1;
    FILE 		*fp;
    int 		i, j, m, type;
    int			xsize, ysize;
	unsigned char	data, *b2;

    xsize = img->size.x;
    ysize = img->size.y;
    m = img->type;
    
    if((fp=fopen(file,"wb")) == NULL) exit( -1 );
    if ( m == 1 ){
		type = 5;	
		fprintf(stderr, "Writing PGM format, %d x %d image : %s\n", xsize, ysize, file);
    } else if( m == 3 ){
		type = 6;
		fprintf(stderr, "Writing PPM format, %d x %d image : %s\n", xsize, ysize, file);
    } else exit( -1 );
    
    fprintf(fp,"P%d\n%d %d\n%d\n", type, xsize, ysize, 255);
    buff = (unsigned char *)calloc( m * xsize * ysize, sizeof( char ) );
    if( buff == NULL ) exit( -1 );

    b2 = img->data;
    b1 = buff;
    for ( i=0; i<ysize; i++ ){
		for ( j=0; j<xsize*m; j++ ){
			data = *b2++ ;
			if (data > 255) data = 255;
			if (data < 0)   data = 0;
			*b1++ = (unsigned char)data;
		}
	}
	fwrite(buff, 1, xsize * ysize * m, fp);
	    
    fclose(fp);
    free(buff);
}

/*=======================================================
  -------------------------------------------------------
  Conversion of RGB format into YUV format
  -------------------------------------------------------
  Argument
    Image *img:     pointer to image memory structure for RGB
    Image *img_yuv: pointer to image memory structure for YUV
  =======================================================*/

static void RGB2YUV(Image *img, Image *im_yuv)
{
	int i, j ;
	float fr, fg, fb ;
	int xsize, ysize;
	xsize = img->size.x;
	ysize = img->size.y;
	for(i = 0; i < ysize; i++){
		for(j = 0; j < xsize; j++){
			fr = (float)img->data[(i*xsize+j)*3] ;		//R
			fg = (float)img->data[(i*xsize+j)*3+1] ;	//G
			fb = (float)img->data[(i*xsize+j)*3+2] ;	//B
			im_yuv->data[(i*xsize+j)*3] = (int)(0.3 * fr + 0.59 * fg + 0.11 * fb) ;	    //Y
			im_yuv->data[(i*xsize+j)*3+1] = (int)(0.7 * fr - 0.59 * fg - 0.11 * fb) ;	//U
			im_yuv->data[(i*xsize+j)*3+2] = (int)(-0.3 * fr - 0.59 * fg + 0.89 * fb) ;	//V
		}
	}
}

typedef struct {
	double intrinsic[3][3];
	double xy[2][1];
	double extrinsic[3][4];
} cparam;
